################################### FUNCTIONS ###################################
loadData =function(filename) {
  dat = read.csv(filename, header=TRUE, sep='\t', fill=FALSE)
  dat$sampleid=rownames(dat)
  dat$patientid=gsub('_NS.*|_BR.*','',dat$sampleid)
  dat$NS = as.integer(dat$smokingStatus=='never')
  dat$CS = as.integer(dat$smokingStatus=='current')
  dat$FS = as.integer(dat$smokingStatus=='ex.more12months')
  dat$FS[dat$smokingStatus=='ex.1_to_12months']=2
  dat$FS[dat$smokingStatus=='ex.less1month']=3
  dat$FSS = as.integer(dat$FS!=0)
  colnames(dat)[colnames(dat)=='gender']='sex'
  colnames(dat)[colnames(dat)=='COPD']='copd'
  colnames(dat)[colnames(dat)=='packsYear']='packsyear'
  dat$smokingstatus = factor(dat$smokingStatus, levels = c('never', 'ex.more12months', 'ex.1_to_12months', 'ex.less1month', 'current'))
  levels(dat$smokingstatus) = c('never','> 12 months','1-12 months', '< 1 month', 'current')
  dat$cancer = factor(dat$cancer)
  levels(dat$cancer) = c('cancer','no cancer')
  dat$copd = factor(dat$copd, levels=c('yes','no'))
  levels(dat$copd) = c('COPD', 'no COPD')
  dat$packsyear = as.character(dat$packsyear)
  dat$packsyear[is.na(dat$packsyear)] = 'none'
  dat$packsyear = factor(dat$packsyear, levels=c('none', 'PYH.0.to.10yrs', 'PYH.11.to.30yrs', 'PYH.more31yrs'))
  levels(dat$packsyear) = c('none', '< 10 years', '10-30 years', '>30 years')
  dat$packsyear[dat$packsyear == 'none' & dat$CS==1] = '10-30 years'
  dat$hvnum = as.integer(grepl('HV', dat$patientid))
  dat$hv = ""
  dat$hv[dat$hvnum==0] = 'symptomatic'
  dat$hv[dat$hvnum==1] = 'healthy'
  dat$hv=factor(dat$hv)
  
  gene.columns = colnames(dat)[grepl('ENSG',colnames(dat))] # for gene-level
  # regulon.columns = colnames(datreg)[-(1523:1539)] # for tf-level
  
  dat2 = melt(dat, measure.vars=gene.columns, value.name='gxp', variable.name='geneid')
  dat2 = dat2[,c('geneid', 'sampleid','patientid','batch','gxp','NS', 'CS', 'FS', 'FSS', 'sex', 'age', 'cancer', 'copd', 'packsyear', 'smokingstatus', 'hv', 'hvnum')]
  
  # For TF-level
  # datreg2 = melt(datreg, measure.vars=regulon.columns, value.name='ract', variable.name='regulonid')
  # datreg2 = datreg2[,c('regulonid', 'sampleid','patientid','batch','ract','NS', 'CS', 'FS', 'FSS', 'sex', 'age', 'cancer', 'copd', 'packsyear', 'smokingstatus', 'hv', 'hvnum')]
  
  dat2$smokingstatusnum = as.integer(dat2$smokingstatus)
  dat2$age = factor(dat2$age, levels=c("(24.9,41.5]", "(41.5,58]", "(58,74.5]", "(74.5,91.1]"), ordered=TRUE)
  dat2$agenum = as.integer(dat2$age)
  dat2$cancernum = as.integer(dat2$cancer=='cancer')
  dat2$copdnum = as.integer(dat2$copd=='COPD')
  dat2$packsyearnum = as.integer(dat2$packsyear)
  dat2$class = 'HV'
  dat2$class[dat2$hvnum==0 & dat2$cancernum==0] = 'SP'
  dat2$class[dat2$hvnum==0 & dat2$cancernum==1] = 'CP'
  dat2$class = factor(dat2$class, levels=c('HV', 'SP', 'CP'), ordered=TRUE)
  dat2$classnum = as.integer(dat2$class)
  return(dat2)
}

loadExpressionData = function(filename.nasal, filename.bronchial) {
  dat.nasal = loadData(filename.nasal)
  dat.bronchial = loadData(filename.bronchial)
  stopifnot(all(colnames(dat.nasal)==colnames(dat.bronchial)))
  dat.nasal$tissue = 'nasal'
  dat.bronchial$tissue = 'bronchial'
  dat = rbind(dat.nasal, dat.bronchial)
  dat$tissue = factor(dat$tissue)
  dat$batch = factor(dat$batch)
  genedat = split(dat[,-1], dat$geneid)
  genes = names(genedat)
  for (g in genes) {
    LMbatch = lm(gxp~batch, data=genedat[[g]])
    LMtissue = lm(gxp~batch*tissue, data=genedat[[g]])
    genedat[[g]]$gxpnobatch = attr(predict(LMbatch, type='terms'), 'constant') + resid(LMbatch)
    genedat[[g]]$gxpnotissue = attr(predict(LMtissue, type='terms'), 'constant') + resid(LMtissue)
    genedat[[g]]$gxpnorm = qqnorm(genedat[[g]]$gxpnobatch, plot=FALSE)$x
    genedat[[g]]$gxpnormnotissue = qqnorm(genedat[[g]]$gxpnotissue, plot=FALSE)$x
    }
  return(genedat)
}

loadActivityData = function(filename) {
  datreg = loadData(filename)
  regulondat = split(datreg[,-1], datreg$regulonid)
  regulons = names(regulondat)
  for (g in regulons) {
    LMbatch = lm(ract~batch, data=regulondat[[g]])
    regulondat[[g]]$ractnobatch = attr(predict(LMbatch, type='terms'), 'constant') + resid(LMbatch)
    regulondat[[g]]$ractnorm = qqnorm(regulondat[[g]]$ractnobatch, plot=FALSE)$x
  }
  return(regulondat)
}

# Plot pathway metascore dynamics
plot_pathway_dynamics <- function(x){
  score <- geneset_score(exprmat, go_list[[x]])
  score_df <- as.data.frame(score)
  rownames(score_df) <- rownames(exprmat)
  score_df$group <- exprmat$group
  score_df$smokingstatus <- exprmat$smokingstatus
  a <- summarySE(score_df, measurevar="score", groupvars=c("smokingstatus","group"))
  ggplot(a, aes(x=smokingstatus, y=score, colour=group)) + 
    geom_errorbar(aes(ymin=score-se, ymax=score+se), width=.1, size=1) +
    geom_line(aes(group=group), size=2) +
    geom_point(size=4) +
    theme_minimal() +
    theme(axis.text.x = element_text(size=44, angle=20),
          axis.text.y = element_text(size=30),
          axis.title.y = element_text(size=44),
          legend.key.size = unit(1,'cm'),
          legend.text = element_text(size = 40),
          legend.title = element_text(size = 40),
          legend.position = 'bottom') +
    scale_color_manual(values = c('#1B9E77', '#E7298A')) +
    ylab(x) +
    xlab('') +
    labs(color='Donor status')
}

determine_gene_classes = function(localdat, covariates = c('FSS','FS','CS'), confounding = c('sex', 'agenum', 'batch'), prior = 'JZS', 	alpha = NULL, modelprior = uniform(), initprobs = 'Uniform', method = 'deterministic', posterior_cutoff=0, filter=NULL) {
	if (is.null(filter)) { ## by default use only nasal samples
		filter = (localdat[[1]]$tissue=='nasal')
	}
	ncov = length(covariates)
	form = as.formula(paste('gxp ~ ', paste(covariates, collapse=' + '), '+', paste(confounding, collapse=' + ')))
	
	models.bayes = lapply(localdat, function(x){
		basLM = bas.lm(form,
				data = x[filter,],
				prior = prior, alpha=alpha, include.always=paste('~ 1 + ', paste(confounding, collapse=' + ')),
				modelprior = modelprior, initprobs = initprobs, method=method)
		return(basLM)
	})
	
	beta.bayes = as.data.frame(t(sapply(models.bayes, function(x){
		c = coef(x)
		return(setNames(c$postmean, c$namesx))
	})))
	
	post = as.data.frame(t(sapply(models.bayes, function(basLM){
		result = summary(basLM, n.models=2^ncov)[c(covariates,'PostProbs'),-1]
		df = melt(as.data.frame(t(result)), measure.vars='PostProbs', value.name='PostProbs')
		agg = aggregate(as.formula(paste('PostProbs ~ ', paste(covariates, collapse = ' + '))), data=df, FUN=sum)
		model.names = apply(agg, 1, function(x){paste(x[1:3], collapse='')})
		model.probs = setNames(agg$PostProbs,model.names)
		model.probs = model.probs[sort(names(model.probs))]
		return(model.probs)
	})))
	
	post2 = data.frame(IR=post[,'101'], SR=post[,'111']+post[,'011'], RR=post[,'001'], CA=post[,'100'] + post[,'110'], NS=post[,'000']+post[,'010'])
	rownames(post2)=rownames(post)
	
	wm = apply(post2,1,which.max)
	post.probs = sapply(1:length(wm),function(i){post2[i,wm[i]]})
	names(post.probs) = rownames(post2)
	
	gene.class.bayes = colnames(post2)[wm]
	names(gene.class.bayes) = rownames(post2)
	table(gene.class.bayes)
	
	genestats = post2
	colnames(genestats) = paste('post_', colnames(genestats), sep='')
	tmp = beta.bayes
	colnames(tmp) = paste('beta_', colnames(tmp), sep='')
	genestats = cbind(genestats, tmp)
	genestats$geneclass = gene.class.bayes
	genestats$maxpost = post.probs

	genestats$geneclass[genestats$maxpos<posterior_cutoff] = 'NS'
	
	return(genestats)
}

################################### LOAD LIBRARIES ###################################
library(reshape2)
library(ggplot2)
library(RColorBrewer)
library(ggpubr)
library(BAS)
library(PRROC)
library(biomaRt)
library(lme4)
library(qvalue)
library(glmnet)
library(gplots)
library(ggalluvial)
library(Rmisc)

################################### RUN MODEL ###################################

## load gene expression/activity data and clinical annotation
LOAD.RDS = TRUE

if (!LOAD.RDS) {
  filename.nasal = 'data/normalized_counts/exprDataAnnotated.tsv'
  filename.bronchial = 'data/normalized_counts/exprDataAnnotatedBR.tsv'
  genedat = loadExpressionData(filename.nasal, filename.bronchial)
  saveRDS(genedat, file='data/genedat_nasal_and_bronchial.RDS')
} else {
  genedat = readRDS('data/genedat_nasal_and_bronchial.RDS')
}
# filter out low variance genes
genedat.var = sapply(genedat, function(x){var(x$gxp[x$tissue=='nasal'], na.rm=TRUE)})
genedat = genedat[(genedat.var>0) & (log(genedat.var) > -4)]

# BAYESIAN MODEL ON HV ONLY
genedat_HV <- lapply(genedat, function(x){
  x <- x[x$tissue=='nasal',]
  x <- x[x$hv=='healthy',]
  return(x)
})
genestats_HV = determine_gene_classes(genedat_HV)
genestats_HV$geneclass_strict <- genestats_HV$geneclass
genestats_HV$geneclass_strict[genestats_HV$geneclass != 'CA' & abs(genestats_HV$beta_CS) < 0.4] <- 'NS'
genestats_HV$geneclass_strict[genestats_HV$geneclass == 'CA' & abs(genestats_HV$beta_FSS) < 0.25] <- 'NS'
write.table(genestats_HV,file="data/smoke_injury/genestats_healthy.tsv",sep="\t",quote=FALSE,row.names=FALSE)

# BAYESIAN MODEL ON CP ONLY
genedat_CP <- lapply(genedat, function(x){
  x <- x[x$tissue=='nasal',]
  xnv <- x[x$hv=='healthy' & x$smokingstatus=='never',]
  xcp <- x[x$hv=='symptomatic' & x$smokingstatus!='never',]
  x <- rbind(xnv, xcp)
  return(x)
})
genestats_CP = determine_gene_classes(genedat_CP) 
genestats_CP$geneclass_strict <- genestats_CP$geneclass
genestats_CP$geneclass_strict[genestats_CP$geneclass != 'CA' & abs(genestats_CP$beta_CS) < 0.4] <- 'NS'
genestats_CP$geneclass_strict[genestats_CP$geneclass == 'CA' & abs(genestats_CP$beta_FSS) < 0.25] <- 'NS'
write.table(genestats_CP,file="data/smoke_injury/genestats_clinic.tsv",sep="\t",quote=FALSE,row.names=FALSE)

################################### PLOT RESULTS ###################################
# Sankey plot
geneclass.crosstab = data.frame(table(Clinic=genestats_CP$geneclass_strict, HV=genestats_HV$geneclass_strict))
geneclass.crosstab = geneclass.crosstab[!(geneclass.crosstab$Clinic=='NS' & geneclass.crosstab$HV=='NS'),]

geneclass.combined = data.frame(Clinic=genestats_CP$geneclass_strict, HV=genestats_HV$geneclass_strict)
geneclass.combined$geneid = names(genedat)

reverse=FALSE

plotdat = to_lodes_form(as.data.frame(geneclass.crosstab), axes=1:2, value = 'geneclass', key = 'donorclass')
plotdat$donorclass = factor(plotdat$donorclass, levels=c('HV', 'Clinic'))
plotdat$geneclass <- as.character(plotdat$geneclass)
plotdat$geneclass[plotdat$geneclass=='NS'] <- 'Unaffected by smoking' 
plotdat$geneclass[plotdat$geneclass=='CA'] <- 'Cessation-associated' 
plotdat$geneclass[plotdat$geneclass=='RR'] <- 'Rapidly reversible' 
plotdat$geneclass[plotdat$geneclass=='SR'] <- 'Slowly reversible' 
plotdat$geneclass[plotdat$geneclass=='IR'] <- 'Irreversible' 
plotdat$geneclass = factor(plotdat$geneclass, levels=c('Unaffected by smoking','Cessation-associated','Irreversible','Slowly reversible','Rapidly reversible'))

ggplot(data=plotdat, aes(x=donorclass, stratum=geneclass, alluvium=alluvium, y=Freq, fill=geneclass)) + 
  geom_flow(reverse=reverse)+
  geom_stratum(reverse=reverse) +
  geom_label(stat = "stratum", aes(label = after_stat(count)), reverse = reverse, size=10, fill='white') +
  xlab('Donor class') +
  ylab('Frequency') +
  labs(fill='Gene class') +
  scale_fill_manual(values = c('grey', '#52a128', '#CD534CFF', '#EFC000FF','#0073C2FF')) +
  theme_minimal() +
  theme(axis.text.x = element_text(size=40),
        axis.text.y = element_blank(),
        axis.title.y = element_blank(),
        axis.title.x = element_text(size=40),
        legend.key.size = unit(1,'cm'),
        legend.text = element_text(size = 32),
        legend.title = element_text(size = 32),
        legend.position = 'right') 

# PCA gene classes HV and CP
exprmat_hv <- sapply(genedat_HV, function(x){return(setNames(x$gxp, x$sampleid))})
exprmat_cp <- sapply(genedat_CP, function(x){return(setNames(x$gxp, x$sampleid))})
sampledat = genedat[[1]][,!grepl('gxp', colnames(genedat[[1]]))]
rownames(sampledat) = sampledat$sampleid
PCA.rr.hv = prcomp(exprmat_hv[,genestats_HV$geneclass_strict=='RR'])
PCA.sr.hv = prcomp(exprmat_hv[,genestats_HV$geneclass_strict=='SR'])
PCA.ir.hv = prcomp(exprmat_hv[,genestats_HV$geneclass_strict=='IR'])
pcadat_hv = data.frame(PC1=c(PCA.rr.hv$x[,1],PCA.sr.hv$x[,1],PCA.ir.hv$x[,1]),
                       PC2=c(PCA.rr.hv$x[,2],PCA.sr.hv$x[,2],PCA.ir.hv$x[,2]),
                       genecat=rep(c('RR','SR','IR'), each=nrow(PCA.rr.hv$x)),
                       sampleid=rep(rownames(PCA.rr.hv$x),3), stringsAsFactors=FALSE)
pcadat_hv$smokingstatus = sampledat[pcadat_hv$sampleid, 'smokingstatus']
#pcadat_hv$genecat <- factor(pcadat_hv$genecat, levels = c('RR','SR','IR'), labels =c('RR','SR','IR'))

plotdat_avg_hv <- pcadat_hv %>% 
  group_by(smokingstatus, genecat) %>% 
  dplyr::summarise(PC1 = mean(PC1),
                   PC2  = mean(PC2))

ggplot(pcadat_hv, aes(x=PC1, y=PC2, color=smokingstatus)) +
  geom_point(size=2,alpha=0.3)  +
  geom_point(data = plotdat_avg_hv,size=8) +
  scale_shape_manual(values = 1) +
  theme_bw() +
  theme(axis.title = element_text(size=20),
        legend.text=element_text(size=20),
        axis.text = element_text(size = 18),
        legend.title = element_text(size = 20),
        strip.text.x = element_text(size = 20),
        legend.position = 'bottom') +
  scale_color_manual(values=c("dodgerblue4", "green4","#E7B800","#FC4E07")) +
  #scale_colour_smoking +
  guides(colour = guide_legend(override.aes = list(size=7))) +
  facet_wrap(~genecat, scales='free', ncol=3)


# STACKED GO TERMS BARPLOTS
# retrieve genes in GO terms
CILIUM_ORGANIZATION = get('GO:0044782', org.Hs.egGO2ALLEGS)
CILIUM_ORGANIZATION = unlist(mget(CILIUM_ORGANIZATION,org.Hs.egENSEMBL))
NEUTROPHIL_MEDIATED_IMMUNITY = get('GO:0002446', org.Hs.egGO2ALLEGS)
NEUTROPHIL_MEDIATED_IMMUNITY = unlist(mget(NEUTROPHIL_MEDIATED_IMMUNITY,org.Hs.egENSEMBL))
INFLAMMATORY_RESPONSE = get('GO:0006954', org.Hs.egGO2ALLEGS)
INFLAMMATORY_RESPONSE = unlist(mget(INFLAMMATORY_RESPONSE,org.Hs.egENSEMBL))
RESPONSE_TO_INTERFERON_GAMMA = get('GO:0034341', org.Hs.egGO2ALLEGS)
RESPONSE_TO_INTERFERON_GAMMA = unlist(mget(RESPONSE_TO_INTERFERON_GAMMA,org.Hs.egENSEMBL))
EXTRACELLULAR_MATRIX_ORGANIZATION = get('GO:0030198', org.Hs.egGO2ALLEGS)
EXTRACELLULAR_MATRIX_ORGANIZATION = unlist(mget(EXTRACELLULAR_MATRIX_ORGANIZATION,org.Hs.egENSEMBL))
XENOBIOTIC_METABOLIC_PROCESS = get('GO:0006805', org.Hs.egGO2ALLEGS)
XENOBIOTIC_METABOLIC_PROCESS = unlist(mget(XENOBIOTIC_METABOLIC_PROCESS,org.Hs.egENSEMBL))
KERATINIZATION = get('GO:0031424', org.Hs.egGO2ALLEGS)
KERATINIZATION = unlist(mget(KERATINIZATION,org.Hs.egENSEMBL))

not_ns_both <- rownames(genestats_HV[!(genestats_HV$geneclass_strict=='NS' & genestats_CP$geneclass_strict=='NS'),])
go_list <- list(
  'Cilium organization' = CILIUM_ORGANIZATION,
  'Keratinization' = KERATINIZATION,
  'Xenobiotic metabolism' = XENOBIOTIC_METABOLIC_PROCESS,
  'Inflammatory response' = INFLAMMATORY_RESPONSE,
  'Extracellular matrix organization' = EXTRACELLULAR_MATRIX_ORGANIZATION,
  'Response to interferon gamma' = RESPONSE_TO_INTERFERON_GAMMA,
  'Neutrophil mediated immunity' = NEUTROPHIL_MEDIATED_IMMUNITY,
  'All genes' = rownames(genestats_HV[not_ns_both,])
)

gene_classes_HV <- genestats_HV[not_ns_both,]
gene_classes_HV$class <- as.factor(gene_classes_HV$geneclass_strict)
gene_classes_CP <- genestats_CP[not_ns_both,]
gene_classes_CP$class <- as.factor(gene_classes_CP$geneclass_strict)

go_table_HV <- lapply(go_list, function(x){
  genes <- intersect(rownames(genestats_HV[not_ns_both,]), x)
  class_freq <- as.vector(table(gene_classes_HV[genes,]$class, useNA='ifany'))
  names(class_freq) <- c('CA','IR','NS','RR','SR')
  return(class_freq)
})
go_table_HV <- as.data.frame(go_table_HV)
go_table_HV$class <- names(table(gene_classes_HV$class))
go_toplot_HV <- gather(go_table_HV,go_term,freq,Cilium.organization:All.genes)
go_toplot_HV$go_term <- factor(go_toplot_HV$go_term,
                               levels=c('Neutrophil.mediated.immunity', 'Xenobiotic.metabolism','Response.to.interferon.gamma','Keratinization',
                                        'Inflammatory.response','Extracellular.matrix.organization','Cilium.organization',
                                        'All.genes'))
go_toplot_HV$model <- 'HV'

tmp <- colSums(go_table_HV[,-ncol(go_table_HV)])
xlabels <- unlist(lapply(tmp, function(x){paste0('(', x, ')')}))
xlabels <- xlabels[order(names(xlabels),decreasing = T)]


gene_classes_CP <- genestats_CP[not_ns_both,]
gene_classes_CP$class <- as.factor(gene_classes_CP$geneclass_strict)
gene_classes_CP <- genestats_CP[not_ns_both,]
gene_classes_CP$class <- as.factor(gene_classes_CP$geneclass_strict)

go_table_CP <- lapply(go_list, function(x){
  genes <- intersect(rownames(genestats_CP[not_ns_both,]), x)
  class_freq <- as.vector(table(gene_classes_CP[genes,]$class, useNA='ifany'))
  names(class_freq) <- c('CA','IR','NS','RR','SR')
  return(class_freq)
})
go_table_CP <- as.data.frame(go_table_CP)
go_table_CP$class <- names(table(gene_classes_CP$class))
go_toplot_CP <- gather(go_table_CP,go_term,freq,Cilium.organization:All.genes)
go_toplot_CP$go_term <- factor(go_toplot_CP$go_term,
                               levels=c('Neutrophil.mediated.immunity', 'Xenobiotic.metabolism','Response.to.interferon.gamma','Keratinization',
                                        'Inflammatory.response','Extracellular.matrix.organization','Cilium.organization',
                                        'All.genes'))
go_toplot_CP$model <- 'Clinic'

tmp <- colSums(go_table_CP[,-ncol(go_table_CP)])
xlabels <- unlist(lapply(tmp, function(x){paste0('(', x, ')')}))
xlabels <- xlabels[order(names(xlabels),decreasing = T)]

go_toplot <- rbind(go_toplot_HV, go_toplot_CP)
go_toplot$model <- factor(go_toplot$model, levels = c('HV','Clinic'))
go_toplot$class <- factor(go_toplot$class, levels=c('NS','RR','SR','IR','CA'))

ggplot(go_toplot, aes(fill=class, y=freq, x=go_term)) +
  geom_bar(position="fill", stat="identity")+
  scale_y_continuous(labels = scales::percent) +
  scale_fill_manual(values = c('grey','#0073C2FF','#EFC000FF','#CD534CFF','#52a128')) +
  coord_flip() +
  theme_minimal()+
  scale_x_discrete(labels=paste(names(xlabels),xlabels, sep = '\n')) +
  theme(axis.text = element_text(size=32),
        axis.title = element_blank(),
        strip.text = element_text(size=40),
        legend.key.size = unit(1,'cm'),
        legend.text = element_text(size=32),
        legend.title = element_blank(),
        legend.position = 'none',
        plot.margin=unit(c(1,1,1.5,1.2),"cm"),
        panel.spacing = unit(4, "lines")) +
  facet_wrap(~model)

# Plot gene dynamics in healthy and clinic
plot_dynamics <- function(x){
  gene_cp <- genedat_CP[[x]]
  gene_cp$group <- 'Clinic'
  gene_hv <- genedat_HV[[x]]
  gene_hv$group <- 'Healthy'
  for_dynamics <- rbind(gene_hv, gene_cp)
  for_dynamics$smokingstatus <- factor(for_dynamics$smokingstatus, levels = c('never','> 12 months', '1-12 months', '< 1 month', 'current'))
  for_dynamics <- for_dynamics[for_dynamics$smokingstatus!='< 1 month',]
  for_dynamics$group <- factor(for_dynamics$group, levels = c('Healthy','Clinic'))
  a <- summarySE(for_dynamics, measurevar="gxpnobatch", groupvars=c("smokingstatus","group"))
  ggplot(a, aes(x=smokingstatus, y=gxpnobatch, colour=group)) + 
    geom_errorbar(aes(ymin=gxpnobatch-se, ymax=gxpnobatch+se), width=.1, size=1) +
    geom_line(aes(group=group), size=2) +
    geom_point(size=4) +
    theme_minimal() +
    theme(axis.text.x = element_text(size=36, angle=20),
          axis.text.y = element_text(size=22),
          axis.title.y = element_text(size=40),
          strip.text =element_text(size=40),
          legend.key.size = unit(1,'cm'),
          legend.text = element_text(size = 36),
          legend.title = element_text(size = 36),
          legend.position = 'bottom') +
    scale_color_manual(values = c('#1B9E77', '#E7298A')) +
    ylab(paste(geneannot[x,]$hgnc_symbol, 'expression', sep = ' ')) +
    xlab('') +
    labs(color='Donor status')
}

plot_dynamics('ENSG00000138061')
plot_dynamics('ENSG00000101144')
plot_dynamics('ENSG00000183347')
plot_dynamics('ENSG00000162543')
