library(glmnet)
library(ggplot2)
library(ggpubr)
library(plyr)
library(dplyr)
library(PRROC)
library(reshape2)



# Function to calculate both clinic and population score
lassoCancer=function(xgenes,sampledat,tissue='nasal',lambda='lambda.min',family='binomial',testCancer=1,testHV=1){
  # Population Score
  if (testHV==1){
   y='hvnum'
   yy = sampledat[sampledat$tissue==tissue, y]
   cvfit = cv.glmnet(y=yy, x=xgenes, alpha=1, nfolds = 50, family=family)
   lambdaFit = cvfit[[lambda]]
   glmcoef = coef(cvfit, s = lambdaFit)
   mygenes = rownames(as.matrix(glmcoef))[as.matrix(glmcoef)[,1]!=0][-1]
   glmpredict.link = predict(cvfit, newx=xgenes, s=lambda, family=family, type='link')
   glmpredict.response = predict(cvfit, newx=xgenes, s=lambdaFit, family=family, type='response')
   lasso.hv.nasal <- list(cvfit=cvfit, predict.link=glmpredict.link, predict.response = glmpredict.response, coef=glmcoef, genes = mygenes)
   
   HVgenes=mygenes
  }else{lasso.hv.nasal=NA}
  
  # Clinic Score
  if (testCancer==1){
   y='cancernum'
   yy = sampledat[sampledat$tissue==tissue, y]
   cvfit = cv.glmnet(y=yy, x=xgenes, alpha=1, nfolds = 50, family=family)
   lambdaFit = cvfit[[lambda]]
   glmcoef = coef(cvfit, s = lambdaFit)
   mygenes = rownames(as.matrix(glmcoef))[as.matrix(glmcoef)[,1]!=0][-1]
   glmpredict.link = predict(cvfit, newx=xgenes, s=lambdaFit, family=family, type='link')
   glmpredict.response = predict(cvfit, newx=xgenes, s=lambdaFit, family=family, type='response')
   lasso.cancer.nasal <- list(cvfit=cvfit, predict.link=glmpredict.link, predict.response = glmpredict.response, coef=glmcoef, genes = mygenes)
   CancerGenes=mygenes
  }else{lasso.cancer.nasal=NA}
  
  results=list("lasso.cancer.nasal"=lasso.cancer.nasal,
               "lasso.hv.nasal"=lasso.hv.nasal)
  return(results)
}


# Load your data first

#GeneDat is the data frame contaiing variance stabilized gene expression.
# genes are in row and subjects in column.
nbToTGene=dim(GeneDat)[1]

#parameters for the Cross Validation
nbGroup=10
nbRand=20
nbRep=10

# Load only genes that have a differenre reversibility class between HV and Clinic patients
listGenes=which(genestatsHV$geneclass_strict != genestatsClinic$geneclass_strict) 
selectGene=rownames(genestatsHV[listGenes,])
nbGenes=length(selectGene)

xGene = as.matrix(exprmat[sampledat$tissue==tissue, selectGene])
xClin=cbind(sampledat$agenum,
            sampledat$smokingstatusnum,
            sampledat$packsyearnum,
            sampledat$sexnum)
colnames(xClin)=c("age","Smoking","PacksYear","sex")
xGeneTot=cbind(xGene,xClin)

# Create lists of random genes
selectGeneRand=sample(1:nbToTGene,nbGenes)
AllselectGeneRand=data.frame("Rand1"=selectGeneRand)
for (rpos in 2:nbRand){
   selectGeneRand=sample(1:nbToTGene,nbGenes)
   AllselectGeneRand[[paste("Rand",rpos,sep='')]]=selectGeneRand
}

#######
#
#  Initialization

auc_pr_cancerGeneRand=matrix(ncol = nbRand,nrow = nbGroup*nbRep)
auc_pr_hvGeneRand=matrix(ncol = nbRand,nrow = nbGroup*nbRep)
auc_roc_cancerGeneRand=matrix(ncol = nbRand,nrow = nbGroup*nbRep)
auc_roc_hvGeneRand=matrix(ncol = nbRand,nrow = nbGroup*nbRep)

auc_pr_cancerGeneRandClin=matrix(ncol = nbRand,nrow = nbGroup*nbRep)
auc_pr_hvGeneRandClin=matrix(ncol = nbRand,nrow = nbGroup*nbRep)
auc_roc_cancerGeneRandClin=matrix(ncol = nbRand,nrow = nbGroup*nbRep)
auc_roc_hvGeneRandClin=matrix(ncol = nbRand,nrow = nbGroup*nbRep)

auc_pr_cancerGene=c()
auc_pr_hvGene=c()
auc_roc_cancerGene=c()
auc_roc_hvGene=c()
auc_pr_cancerGeneClin=c()
auc_pr_hvGeneClin=c()
auc_roc_cancerGeneClin=c()
auc_roc_hvGeneClin=c()
auc_pr_cancerClin=c()
auc_pr_hvClin=c()
auc_roc_cancerClin=c()
auc_roc_hvClin=c()


lassoPerGroupGeneRand=list()
lassoPerGroupGeneRandClin=list()
lassoPerGroupGene=list()
lassoPerGroupGeneClin=list()
lassoPerGroupClin=list()

StoreROCPR=list()
StoreROCPR[["ROC"]]=list()
StoreROCPR[["PR"]]=list()

for (k in c("ROC","PR")){
   for (i in c("Clin","Gene","GeneClin")){
      StoreROCPR[[paste(k)]][[paste(i)]]=list()
      for (j in c("Cancer","hv")){
         StoreROCPR[[paste(k)]][[paste(i)]][[paste(j)]]=list()
      }
   }
}


lassoPerGroup=list()
lassoPerGroup$Clin=list()
lassoPerGroup$GeneClin=list()
lassoPerGroup$Gene=list()
for (rpos in 1:nbRand){
   lassoPerGroup[[paste("RandGene",rpos,sep='')]]=list()
}



for (j in c(1:nbRep)){
   
   # construct test and train
   rr=runif(dim(sampledat)[1])
   groups_thr=quantile(rr,seq(0,1,1/nbGroup),names=F)
   groups=rep(1,dim(sampledat)[1])
   
   for (i in 2:nbGroup){
      groups[which(rr>groups_thr[i])]=i
   }
   sampledat$groups=groups
   
   
   for (u in c(1:nbGroup)){
      
      thisRep=nbGroup*(j-1)+u
      test = which(sampledat$groups == u)
      train=which(sampledat$groups !=  u)
      
      # Training: 
      
      #Calculate the scores for the different models
      #using just gene expression
      resultsLassoGene=lassoCancer(xGene[train,],sampledat = sampledat[train,]  )
      lassoPerGroup$Gene[[paste("group",thisRep,sep='')]]=resultsLassoGene
      #using  gene expression and clinical info
      resultsLassoGeneClin=lassoCancer(xGeneTot[train,],sampledat = sampledat[train,]  )
      lassoPerGroup$GeneClin[[paste("group",thisRep,sep='')]]=resultsLassoGeneClin
      #using just clinical info
      resultsLassoClin=lassoCancer(xClin[train,],sampledat = sampledat[train,]  )
      lassoPerGroup$Clin[[paste("group",thisRep,sep='')]]=resultsLassoClin
      
      #Test, Clinic score
      newxGeneTot=xGeneTot[test,]
      newxGene=xGene[test,]
      newxClin=xClin[test,]
      newy=sampledat[test,]$cancernum
      
      #Predict Gene
      cvfit=resultsLassoGene$lasso.cancer.nasal$cvfit
      lambdaFit = cvfit[["lambda.min"]]
      glmpredict.response.Gene = predict(cvfit, newx=newxGene, s=lambdaFit, family=family, type='response')
      
      #Predict Gene+Clin
      cvfit=resultsLassoGeneClin$lasso.cancer.nasal$cvfit
      lambdaFit = cvfit[["lambda.min"]]
      glmpredict.response.GeneClin = predict(cvfit, newx=newxGeneTot, s=lambdaFit, family=family, type='response')
      
      #Predict Clinic only
      cvfit=resultsLassoClin$lasso.cancer.nasal$cvfit
      lambdaFit = cvfit[["lambda.min"]]
      glmpredict.response.Clin = predict(cvfit, newx=newxClin, s=lambdaFit, family=family, type='response')
      
      fg <- glmpredict.response.Gene[newy == 1]
      bg <- glmpredict.response.Gene[newy == 0]
      pr <- pr.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
      roc <- roc.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
      auc_pr_cancerGene[thisRep]=pr$auc.integral
      auc_roc_cancerGene[thisRep]=roc$auc
      StoreROCPR$ROC$Gene[["Cancer"]][[paste("group",thisRep,sep='')]]=roc
      StoreROCPR$PR$Gene[["Cancer"]][[paste("group",thisRep,sep='')]]=pr
      
      fg <- glmpredict.response.GeneClin[newy == 1]
      bg <- glmpredict.response.GeneClin[newy == 0]
      pr <- pr.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
      roc <- roc.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
      auc_pr_cancerGeneClin[thisRep]=pr$auc.integral
      auc_roc_cancerGeneClin[thisRep]=roc$auc
      StoreROCPR$ROC$GeneClin[["Cancer"]][[paste("group",thisRep,sep='')]]=roc
      StoreROCPR$PR$GeneClin[["Cancer"]][[paste("group",thisRep,sep='')]]=pr
      
      fg <- glmpredict.response.Clin[newy == 1]
      bg <- glmpredict.response.Clin[newy == 0]
      pr <- pr.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
      roc <- roc.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
      auc_pr_cancerClin[thisRep]=pr$auc.integral
      auc_roc_cancerClin[thisRep]=roc$auc
      StoreROCPR$ROC$Clin[["Cancer"]][[paste("group",thisRep,sep='')]]=roc
      StoreROCPR$PR$Clin[["Cancer"]][[paste("group",thisRep,sep='')]]=pr
      
      #Test, Population score
      newy=sampledat[test,]$hvnum
      

      cvfit=resultsLassoGene$lasso.hv.nasal$cvfit
      lambdaFit = cvfit[["lambda.min"]]
      glmpredict.response.Gene = predict(cvfit, newx=newxGene, s=lambdaFit, family=family, type='response')
      cvfit=resultsLassoGeneClin$lasso.hv.nasal$cvfit
      lambdaFit = cvfit[["lambda.min"]]
      glmpredict.response.GeneClin = predict(cvfit, newx=newxGeneTot, s=lambdaFit, family=family, type='response')
      cvfit=resultsLassoClin$lasso.hv.nasal$cvfit
      lambdaFit = cvfit[["lambda.min"]]
      glmpredict.response.Clin = predict(cvfit, newx=newxClin, s=lambdaFit, family=family, type='response')
      
      fg <- glmpredict.response.Gene[newy == 1]
      bg <- glmpredict.response.Gene[newy == 0]
      pr <- pr.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
      roc <- roc.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
      auc_pr_hvGene[thisRep]=pr$auc.integral
      auc_roc_hvGene[thisRep]=roc$auc
      StoreROCPR$ROC$Gene[["hv"]][[paste("group",thisRep,sep='')]]=roc
      StoreROCPR$PR$Gene[["hv"]][[paste("group",thisRep,sep='')]]=pr
      
      fg <- glmpredict.response.GeneClin[newy == 1]
      bg <- glmpredict.response.GeneClin[newy == 0]
      pr <- pr.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
      roc <- roc.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
      auc_pr_hvGeneClin[thisRep]=pr$auc.integral
      auc_roc_hvGeneClin[thisRep]=roc$auc
      StoreROCPR$ROC$GeneClin[["hv"]][[paste("group",thisRep,sep='')]]=roc
      StoreROCPR$PR$GeneClin[["hv"]][[paste("group",thisRep,sep='')]]=pr
      
      fg <- glmpredict.response.Clin[newy == 1]
      bg <- glmpredict.response.Clin[newy == 0]
      pr <- pr.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
      roc <- roc.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
      auc_pr_hvClin[thisRep]=pr$auc.integral
      auc_roc_hvClin[thisRep]=roc$auc
      StoreROCPR$ROC$Clin[["hv"]][[paste("group",thisRep,sep='')]]=roc
      StoreROCPR$PR$Clin[["hv"]][[paste("group",thisRep,sep='')]]=pr
      
      #Test and train for the random models
      
      for (rpos in 1:nbRand){
         
         #Train
         xGeneRand = as.matrix(exprmat[sampledat$tissue==tissue, AllselectGeneRand[[paste("Rand",rpos,sep='')]]])
         xGeneRandTot=cbind(xGeneRand,xClin)
         newxGeneRand=xGeneRand[test,]
         newxGeneRandTot=xGeneRandTot[test,]
         resultsLassoGeneRand=lassoCancer(xGeneRand[train,],sampledat = sampledat[train,] )
         lassoPerGroup[[paste("RandGene",rpos,sep='')]][[paste("group",thisRep,sep='')]]=resultsLassoGeneRand
         resultsLassoGeneRandClin=lassoCancer(xGeneRandTot[train,],sampledat = sampledat[train,] )
         lassoPerGroup[[paste("RandGeneClin",rpos,sep='')]][[paste("group",thisRep,sep='')]]=resultsLassoGeneRand
         
         #Test clinic Score
         
         "Predict GeneRand"
         newy=sampledat[test,]$cancernum
         
         cvfit=resultsLassoGeneRand$lasso.cancer.nasal$cvfit
         lambdaFit = cvfit[["lambda.min"]]
         glmpredict.response.GeneRand = predict(cvfit, newx=newxGeneRand, s=lambdaFit, family=family, type='response')
         
         fg <- glmpredict.response.GeneRand[newy == 1]
         bg <- glmpredict.response.GeneRand[newy == 0]
         pr <- pr.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
         roc <- roc.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
         auc_pr_cancerGeneRand[thisRep,rpos]=pr$auc.integral
         auc_roc_cancerGeneRand[thisRep,rpos]=roc$auc
         
         cvfit=resultsLassoGeneRandClin$lasso.cancer.nasal$cvfit
         lambdaFit = cvfit[["lambda.min"]]
         glmpredict.response.GeneRandClin = predict(cvfit, newx=newxGeneRandTot, s=lambdaFit, family=family, type='response')
         
         fg <- glmpredict.response.GeneRandClin[newy == 1]
         bg <- glmpredict.response.GeneRandClin[newy == 0]
         pr <- pr.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
         roc <- roc.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
         auc_pr_cancerGeneRandClin[thisRep,rpos]=pr$auc.integral
         auc_roc_cancerGeneRandClin[thisRep,rpos]=roc$auc
         
         # Test Pop Score
         newy=sampledat[test,]$hvnum
         cvfit=resultsLassoGeneRand$lasso.hv.nasal$cvfit
         lambdaFit = cvfit[["lambda.min"]]
         glmpredict.response.GeneRand = predict(cvfit, newx=newxGeneRand, s=lambdaFit, family=family, type='response')
         fg <- glmpredict.response.GeneRand[newy == 1]
         bg <- glmpredict.response.GeneRand[newy == 0]
         pr <- pr.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
         roc <- roc.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
         auc_pr_hvGeneRand[thisRep,rpos]=pr$auc.integral
         auc_roc_hvGeneRand[thisRep,rpos]=roc$auc
         
         cvfit=resultsLassoGeneRandClin$lasso.hv.nasal$cvfit
         lambdaFit = cvfit[["lambda.min"]]
         glmpredict.response.GeneRandClin = predict(cvfit, newx=newxGeneRandTot, s=lambdaFit, family=family, type='response')
         
         fg <- glmpredict.response.GeneRandClin[newy == 1]
         bg <- glmpredict.response.GeneRandClin[newy == 0]
         pr <- pr.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
         roc <- roc.curve(scores.class0 = fg, scores.class1 = bg, curve = T)
         auc_pr_hvGeneRandClin[thisRep,rpos]=pr$auc.integral
         auc_roc_hvGeneRandClin[thisRep,rpos]=roc$auc
         
      }
      
   }
}


AUCtable=data.frame("CancerGenePR"=auc_pr_cancerGene,
                    "hvGenePR"=auc_pr_hvGene,
                    "CancerGeneROC"=auc_roc_cancerGene,
                    "hvGeneROC"=auc_roc_hvGene,
                    "CancerGeneClinPR"=auc_pr_cancerGeneClin,
                    "hvGeneClinPR"=auc_pr_hvGeneClin,
                    "CancerGeneClinROC"=auc_roc_cancerGeneClin,
                    "hvGeneClinROC"=auc_roc_hvGeneClin,
                    "CancerClinPR"=auc_pr_cancerClin,
                    "hvClinPR"=auc_pr_hvClin,
                    "CancerClinROC"=auc_roc_cancerClin,
                    "hvClinROC"=auc_roc_hvClin)
for (rpos in 1:nbRand){
   AUCtable[[paste("CancerGeneRand",rpos,"PR",sep='')]]=auc_pr_cancerGeneRand[1:(j*nbGroup),rpos]
   AUCtable[[paste("CancerGeneRand",rpos,"ROC",sep='')]]=auc_roc_cancerGeneRand[1:(j*nbGroup),rpos]
   AUCtable[[paste("hvGeneRand",rpos,"PR",sep='')]]=auc_pr_hvGeneRand[1:(j*nbGroup),rpos]
   AUCtable[[paste("hvGeneRand",rpos,"ROC",sep='')]]=auc_pr_hvGeneRand[1:(j*nbGroup),rpos]
   AUCtable[[paste("CancerGeneRandClin",rpos,"PR",sep='')]]=auc_pr_cancerGeneRandClin[1:(j*nbGroup),rpos]
   AUCtable[[paste("CancerGeneRandClin",rpos,"ROC",sep='')]]=auc_roc_cancerGeneRandClin[1:(j*nbGroup),rpos]
   AUCtable[[paste("hvGeneRandClin",rpos,"PR",sep='')]]=auc_pr_hvGeneRandClin[1:(j*nbGroup),rpos]
   AUCtable[[paste("hvGeneRandClin",rpos,"ROC",sep='')]]=auc_pr_hvGeneRandClin[1:(j*nbGroup),rpos]
}




Span=0.6


# Script to generate Average ROC and PR curves from all Cross validation sets

for (type in c("ROC","PR")){
   if (type=="PR"){
      xaxis="Precision"
      yaxis="Recall"
   }else if(type=="ROC"){
      xaxis="False Positive Rate"
      yaxis="Sensitivity"
   }
   for (Feat in c("hv")){
      data=data.frame()
      for (Model in c("GeneClin","Clin")){
         for (j in 1:(nbGroup*nbRep)){
            tempdat=data.frame(StoreROCPR[[type]][[Model]][[Feat]][[paste("group",j,sep="")]]$curve)
            tempdat$Group=paste("group",j,sep="")
            tempdat$Model=Model
            data=rbind(data,tempdat)
         }      
         data$Group=as.factor(data$Group)
      }
      if (Feat =="hv") {data$X3 = -data$X3}
      data1=data[which(data$Model=="GeneClin"),]
      data2=data[which(data$Model=="Clin"),]
      data1$predict <- predict(loess(X2~X1, data = data1,span = Span))
      data2$predict <- predict(loess(X2~X1, data = data2,span=Span))
      data1$predictT <- predict(loess(X3~X1, data = data1,span = Span))
      data2$predictT <- predict(loess(X3~X1, data = data2,span=Span))
      data=rbind(data1,data2)
      data$Model=as.factor(data$Model)
      if (Feat=="hv"){Feat2="Population Test"}else if (Feat=="Cancer"){Feat2="Clinical Test"}
      p= ggplot(data, aes(x = X1, y = X2,group=Model)) +
         geom_line(aes(group = Group), alpha = 0) +
         scale_colour_gradientn(colours=rainbow(10))+
         geom_smooth(size = 0.8, span = Span,color=NA,method = "loess")+
         theme_minimal()+
         theme(text = element_text(size = 18),axis.text.x = element_text(angle = 30),
               legend.position = "bottom",legend.text=element_text(size=14))+
         geom_line(aes(x=X1,y=predict,color=predictT))+
         geom_point(data = data[seq(1,dim(data)[1],75),],aes(x=X1,y=predict,shape=Model,
                                                             color=predictT)
                    ,alpha=1,size=4)+
         scale_shape_manual(values=c(17,15))+
         labs(x=xaxis,y=yaxis,title=paste(type," Curve -- ",Feat2,sep=''),colour="Score Threshold")
      
      if (type =="ROC"){
         p=p+geom_hline(yintercept = 0.95,linetype="dashed")
      }      
      print(p)
   
   }
}
dev.off() 


#Save the results
ResultFileAUC="~/AUCtable.txt"
ResulFileLassoModels="~/LassoRiskDistribution.RDS"

saveRDS(lassoPerGroup,file = ResulFileLassoModels)
write.table(file=ResultFile,AUCtable,sep="\t")








