########################################## FUNCTIONS ##########################################
# calculate geneset metascore
geneset_score <- function(exprmat, geneset){
  expr <- exprmat[,colnames(exprmat)%in%geneset]
  score <- rowMeans(expr)
}
# plot dynamics of pathway metascore in healthy and clinic
plot_pathway_dynamics <- function(x){
  score <- geneset_score(exprmat, go_list[[x]])
  score_df <- as.data.frame(score)
  rownames(score_df) <- rownames(exprmat)
  score_df$group <- exprmat$group
  score_df$smokingstatus <- exprmat$smokingstatus
  a <- summarySE(score_df, measurevar="score", groupvars=c("smokingstatus","group"))
  ggplot(a, aes(x=smokingstatus, y=score)) + 
    geom_errorbar(aes(ymin=score-se, ymax=score+se), width=.1, size=1, col='black') +
    geom_line(aes(group=group, linetype=group), size=2, col='black') +
    geom_point(aes(shape=group),size=8, col='black') +
    theme_minimal() +
    theme(axis.text.x = element_text(size=50, angle=20),
          axis.text.y = element_text(size=30),
          axis.title.y = element_text(size=44),
          legend.key.size = unit(1,'cm'),
          legend.text = element_text(size = 40),
          legend.title = element_text(size = 40),
          legend.position = 'none') +
    ylab(x) +
    xlab('') +
    labs(color='Donor status')
}

########################################## LOAD LIBRARIES ##########################################
library(ggplot2)
library(biomaRt)
library(Rmisc)

########################################## GENESET METASCORE ##########################################
# Read-in normalized expression and clinical info
genedat = readRDS('data/genedat_nasal_and_bronchial.RDS') # list of dataframes generated with the loadExpressionData function in 3.SmokeInjuryReversibility
genedat.var = sapply(genedat, function(x){var(x$gxp[x$tissue=='nasal'], na.rm=TRUE)})
genedat = genedat[(genedat.var>0) & (log(genedat.var) > -4)]
genedat_HV <- lapply(genedat, function(x){
  x <- x[x$tissue=='nasal',]
  x <- x[x$hv=='healthy',]
  return(x)
})
genedat_CP <- lapply(genedat, function(x){
  x <- x[x$tissue=='nasal',]
  xnv <- x[x$hv=='healthy' & x$smokingstatus=='never',]
  xcp <- x[x$hv=='symptomatic' & x$smokingstatus!='never',]
  x <- rbind(xnv, xcp)
  return(x)
})

exprmat_hv = as.data.frame(sapply(genedat_HV, function(x){return(setNames(x$gxpnobatch, x$sampleid))}))
exprmat_cp = as.data.frame(sapply(genedat_CP, function(x){return(setNames(x$gxpnobatch, x$sampleid))}))
exprmat_hv <- cbind(exprmat_hv, genedat_HV[[1]])
exprmat_cp <- cbind(exprmat_cp, genedat_CP[[1]])
exprmat_hv$group <- 'HV'
exprmat_cp$group <- 'Clinic'
exprmat <- rbind(exprmat_hv, exprmat_cp)
exprmat$smokingstatus <- factor(exprmat$smokingstatus, levels = c('never','> 12 months', '1-12 months', '< 1 month', 'current'))
exprmat <- exprmat[exprmat$smokingstatus!='< 1 month',]
exprmat$group <- factor(exprmat$group, levels = c('HV','Clinic'))

# retrieve genes in GO terms
CILIUM_ORGANIZATION = get('GO:0044782', org.Hs.egGO2ALLEGS)
CILIUM_ORGANIZATION = unlist(mget(CILIUM_ORGANIZATION,org.Hs.egENSEMBL))
NEUTROPHIL_MEDIATED_IMMUNITY = get('GO:0002446', org.Hs.egGO2ALLEGS)
NEUTROPHIL_MEDIATED_IMMUNITY = unlist(mget(NEUTROPHIL_MEDIATED_IMMUNITY,org.Hs.egENSEMBL))
INFLAMMATORY_RESPONSE = get('GO:0006954', org.Hs.egGO2ALLEGS)
INFLAMMATORY_RESPONSE = unlist(mget(INFLAMMATORY_RESPONSE,org.Hs.egENSEMBL))
RESPONSE_TO_INTERFERON_GAMMA = get('GO:0034341', org.Hs.egGO2ALLEGS)
RESPONSE_TO_INTERFERON_GAMMA = unlist(mget(RESPONSE_TO_INTERFERON_GAMMA,org.Hs.egENSEMBL))
EXTRACELLULAR_MATRIX_ORGANIZATION = get('GO:0030198', org.Hs.egGO2ALLEGS)
EXTRACELLULAR_MATRIX_ORGANIZATION = unlist(mget(EXTRACELLULAR_MATRIX_ORGANIZATION,org.Hs.egENSEMBL))
XENOBIOTIC_METABOLIC_PROCESS = get('GO:0006805', org.Hs.egGO2ALLEGS)
XENOBIOTIC_METABOLIC_PROCESS = unlist(mget(XENOBIOTIC_METABOLIC_PROCESS,org.Hs.egENSEMBL))
KERATINIZATION = get('GO:0031424', org.Hs.egGO2ALLEGS)
KERATINIZATION = unlist(mget(KERATINIZATION,org.Hs.egENSEMBL))
ANTIGEN_PROCESSING_AND_PRESENTATION = get('GO:0019882', org.Hs.egGO2ALLEGS)
ANTIGEN_PROCESSING_AND_PRESENTATION = unlist(mget(ANTIGEN_PROCESSING_AND_PRESENTATION,org.Hs.egENSEMBL))

go_list <- list(
  'Cilium organization' = CILIUM_ORGANIZATION,
  'Keratinization' = KERATINIZATION,
  'Xenobiotic metabolism' = XENOBIOTIC_METABOLIC_PROCESS,
  'Inflammatory response' = INFLAMMATORY_RESPONSE,
  'Extracellular matrix organization' = EXTRACELLULAR_MATRIX_ORGANIZATION,
  'Response to interferon gamma' = RESPONSE_TO_INTERFERON_GAMMA,
  'Neutrophil mediated immunity' = NEUTROPHIL_MEDIATED_IMMUNITY,
  'Antigen processing and presentation' = ANTIGEN_PROCESSING_AND_PRESENTATION)

# calculate metascore
geneset_score <- function(exprmat, geneset){
  expr <- exprmat[,colnames(exprmat)%in%geneset]
  score <- rowMeans(expr)
}
pathway_scores <- lapply(go_list, function(x){geneset_score(exprmat, x)})
pathway_scores <- as.data.frame(pathway_scores)
rownames(pathway_scores) <- rownames(exprmat)
pathway_scores$group <- exprmat$group
pathway_scores$smokingstatus <- exprmat$smokingstatus

p1 <- plot_pathway_dynamics('Response to interferon gamma')
p2 <- plot_pathway_dynamics('Neutrophil mediated immunity')
p3 <- plot_pathway_dynamics('Inflammatory response')
p4 <- plot_pathway_dynamics('Antigen processing and presentation')
p5 <- plot_pathway_dynamics('Cilium organization')
p6 <- plot_pathway_dynamics('Xenobiotic metabolism')
p7 <- plot_pathway_dynamics('Extracellular matrix organization')
p8 <- plot_pathway_dynamics('Keratinization')

ggarrange(p5,p6,p7,p8, p1,p2,p3,p4, ncol=4, nrow=2)