library(MatrixEQTL)

args = commandArgs(trailingOnly=TRUE)
if (length(args) < 1 | args[1] == '--help'){
  message("usage: run_meqtl.R snp_genotype.tsv phenotype.tsv covariates.tsv output_directory [snp_positions.tsv phenotype_positions.tsv] [--cisonly]")
  stop()
}

paste("args[1] : ",args[1],sep='')
paste("args[2] : ",args[2],sep='')
paste("args[3] : ",args[3],sep='')
paste("args[4] : ",args[4],sep='')
paste("args[5] : ",args[5],sep='')
paste("args[6] : ",args[6],sep='')
paste("args[7] : ",args[7],sep='')
paste("args[8] : ",args[8],sep='')
paste("args[9] : ",args[9],sep='')
paste("args[10] : ",args[10],sep='')
paste("args[11] : ",args[11],sep='')



geno_file=args[1]
pheno_file=args[2]
nb_cov=as.numeric(args[3])  # Nb of PCA components
Tissue=args[4] # Select Nasal and Bronchial samples
pval_cis = as.numeric(args[5]) # The pvalue threshold
qqN=args[6] # qq norm or not
out_dir=args[7] # output directory
linear=args[8] # simple test or interaction test
pca_file=args[9] # The file containing the result of the PCA on the expression data

 
cis=T
cisonly = T
pval_trans = 0
nb_cov2=nb_cov+4
maf_thr=0.01 # Threshold for the maf filtering

# Load a genotype and a Phenotype file containing the SNP and vst data respectively
load(pheno_file) #Pheenotype matrix
load(geno_file)  #Genotype matrix
load(pca_file)   
#mat_cov file: contains all the covariates: Tissue / Sex / Age / Smoking Status/ PackYears  => Tissue has to be the first column
#+ The loading on the X first PCA coordinates
# Smoking status has to be encoded "0/1/2/3/4" 
#0: Never smoker, 
#1:  ex >1year, 
#2: ex <1 year
#3: ex < 1month,
#4: Current Smokers
# and packyears 0/1/2/3 
# 0: Never smokers
# 1: packyears<10
# 2: packyears <30
# 3: pack years >30

genotypes = SlicedData$new()
genotypes$CreateFromMatrix(genotypes_matrix)
genotypes$fileOmitCharacters = "NA"

# removing SNP whose MAF<1% and
maf.list = vector('list', length(genotypes))
for(sl in 1:length(genotypes)) {
  slice = genotypes[[sl]];
  maf.list[[sl]] = rowMeans(slice,na.rm=TRUE)/2;
  maf.list[[sl]] = pmin(maf.list[[sl]],1-maf.list[[sl]]);
}
maf = unlist(maf.list)

cat('SNPs before filtering:',nrow(genotypes))
genotypes$RowReorder(maf>maf_thr)
cat('SNPs after filtering:',nrow(genotypes))


# genomic window used for cis/trans eQTL analysis
window = 5*10^5
# Linear model to use
errorCovariance = numeric()
#store lm results:
lmRes=list()
lmNoInt=list()
me = NA
useModel=modelLINEAR
covars$CreateFromMatrix(mat_cov)

if (qqN=="qqnorm"){
  matqq=phenotypes[[1]]
  matqq = t(apply(matqq, 1, rank, ties.method = "average"));
  matqq = qnorm(matqq / (ncol(matqq)+1));
  phenotypes$CreateFromMatrix(as.matrix(matqq))
}else if (qqN=="noqqnorm"){
  mat=pheno_mat
}


outfile = paste0(out_dir,'/Matrix_eQTL_analysis.txt')
outfile_trans=paste(outfile,"trans",sep='_')
outfile_cis=paste(outfile,"cis",sep='_')


genepos = read.table("~/data/all_genes_pos.tsv")
#this file is a 4 column tsv file containing the position of the gene:
# Gene Name / Chromosome / Start of the Gene (bp) / End of the Gene

if (Tissue=="Nasal"){  
  # Select only nasal samples
  genotypes$ColumnSubsample(which(covars[[1]][1,]==1))
  phenotypes$ColumnSubsample(which(covars[[1]][1,]==1))
  covarsPeer$ColumnSubsample(which(covars[[1]][1,]==1))
  #remove Tissue line:
  NewCov=covars[[1]]
  NewCov=NewCov[-1,]
  covars$CreateFromMatrix(NewCov)
}else if (Tissue == "Bronchial" ) {
  # Select only Bronchial samples
  genotypes$ColumnSubsample(which(covars[[1]][1,]==0))
  phenotypes$ColumnSubsample(which(covars[[1]][1,]==0))
  covarsPeer$ColumnSubsample(which(covars[[1]][1,]==0))
  #remove Tissue line:
  NewCov=covars[[1]]
  NewCov=NewCov[-1,]
  covars$CreateFromMatrix(NewCov)
}

if (qqN=="qqnorm"){
    matqq=phenotypes[[1]]
    matqq = t(apply(matqq, 1, rank, ties.method = "average"));
    matqq = qnorm(matqq / (ncol(matqq)+1));
    phenotypes$CreateFromMatrix(as.matrix(matqq))
}

#remove 0 variance covariates:
NewCov=covars[[1]]

v=apply(NewCov,1, function(x) var(x))
toRm=which(v==0)
if (length(toRm)>0){
  NewCov=NewCov[-toRm,]
}
covars$CreateFromMatrix(NewCov)



############


me = Matrix_eQTL_main(
  snps = genotypes,
  gene = phenotypes,
  # cvrt = covarsNoEx,
  cvrt = covars,
  output_file_name = outfile_trans,
  pvOutputThreshold = pval_trans,
  useModel = useModel,
  errorCovariance = errorCovariance,
  verbose = TRUE,
  output_file_name.cis = outfile_cis,
  pvOutputThreshold.cis = pval_cis,
  snpspos = snp_pos,
  genepos = genepos,
  cisDist = window,
  pvalue.hist = "qqplot",
  min.pv.by.genesnp = TRUE,
  noFDRsaveMemory = FALSE)


cat('Analysis done in: ', me$time.in.sec / 60 , ' minutes', '\n')
pdf(paste(outfile,"_qqplot.pdf",sep=""))
plot(me,pch=20,cex=0.5)
warnings()
dev.off()




