# README #

This repository describes the analysis conducted in the paper entitled "Smoking-dependent expression alterations in nasal epithelium reveal immune impairment linked to germline variation and lung cancer risk".
The RNA-seq data was aligned to the human genome version GRCh37 using TopHat2. Raw read counts were obtained using featureCounts.

We provide 8 files to reproduce the main analyses of the paper:

### 0. Read count normalization ###

In this script we show how we normalize the raw read counts obtained with featureCounts using the variance-stabilising transformation from DESeq2.

### 1. Differential expression analysis and variant component analysis ###

Between clinic patients and healthy subjects, and between clinic cancer and clinic benign in nasal and bronchial tissue.
Results of this script are discussed in the section "Main contributors to gene expression in the airway epithelium".
The variant component analysis is represented in Fig.2a.

### 2. Reconstruction of the Transcription Factor network and calculation of the transcription factor activity matrix ###

This script, together with the run_aracne.sh file, is used to construct the TF network used in Fig.3 a and b and for Fig.7.

### 3. Bayesian reversibility analysis of the smoke injury reponse ###

This script can be applied either on the gene expression data, or on the TF activity to calculate the reversibility of the injury either at the gene or at the TF level
(results displayed in Fig.2 b-d and 3 b).

### 4. Estimation of the risk score ###

This script calculates both the clinic and population score, and the cross-validation procedure to estimate the test error. (results displayed in Fig.4).

### 5. Calculation of pathway metascores ###

For the 8 pathways of interest. These scores are used for Fig.3c and Fig.5 b-c.

### 6. Overrepresentation of genes in the target genes of certain TF

This script is used to find the 4 TFs displayed on Fig.7 and to calculate the pvalues and overrepresentation displayed on table 2 and 3.

### 7. eQTL ###

This script can be used to identify SNPs that impact the expression of genes in their vicinity for the eQTL analysis.

### Necessary Packages ###
R packages:  
MatrixEQTL  
glmnet  
ggplot2  
ggpubr  
plyr  
dplyr  
PRROC  
reshape2  
biomaRt
clusterProfiler
BAS
viper
variancePartition



### Who do I talk to? ###

For more informations, contact:  
Stella de Biase: Stella.deBiase@mdc-berlin.de  
 Florian Massip: florian.massip@mines-paristech.fr and  
Roland Schwarz: Roland.Schwarz@mdc-berlin.de
